import datetime

import pytest
from django.utils import timezone
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APIRequestFactory, force_authenticate

from greg.models import Identity, Person, Role, Invitation, InvitationLink
from gregui.api.views.invitation import InvitationView


@pytest.mark.django_db
def test_invite_guest(client, user_sponsor, unit_foo, role_type_foo, mocker):
    # Mock function to avoid exception because there are no e-mail templates in the database
    mocker.patch("gregui.api.views.invitation.send_invite_mail")

    test_comment = "This is a test comment"
    contact_person_unit = "This is a test contact person"
    role_start_date = datetime.datetime.today() + datetime.timedelta(days=1)
    role_end_date = datetime.datetime.today() + datetime.timedelta(days=10)

    data = {
        "first_name": "Foo",
        "last_name": "Bar",
        "email": "test@example.com",
        "role": {
            "start_date": (role_start_date).strftime("%Y-%m-%d"),
            "end_date": (role_end_date).strftime("%Y-%m-%d"),
            "orgunit": unit_foo.id,
            "type": role_type_foo.id,
            "comments": test_comment,
            "contact_person_unit": contact_person_unit,
        },
    }
    url = reverse("gregui-v1:invitation")

    assert len(Person.objects.all()) == 0

    request = APIRequestFactory().post(url, data, format="json")
    force_authenticate(request, user=user_sponsor)

    response = InvitationView.as_view()(request)

    assert response.status_code == status.HTTP_201_CREATED

    assert Person.objects.count() == 1
    person = Person.objects.first()
    assert person.first_name == "Foo"
    assert person.last_name == "Bar"

    assert Identity.objects.filter(
        person=person,
        type=Identity.IdentityType.PRIVATE_EMAIL,
        value="test@example.com",
    ).exists()

    role = Role.objects.filter(person__id=person.id).get()
    assert role.orgunit == unit_foo
    assert role.type == role_type_foo
    assert role.start_date == role_start_date.date()
    assert role.end_date == role_end_date.date()
    assert role.contact_person_unit == contact_person_unit
    assert role.comments == test_comment


@pytest.mark.django_db
def test_invite_cancel(client, invitation_link, invitation, role, log_in, user_sponsor):
    # TODO: Should all sponsors be allowed to delete arbitrary invitations?
    log_in(user_sponsor)
    url = reverse("gregui-v1:invitation")

    # Check that the role is there
    role = Role.objects.get(id=role.id)
    response = client.delete("%s?person_id=%s" % (url, str(role.person_id)))

    assert response.status_code == status.HTTP_200_OK

    # The role, invitation and connected links should now have been removed
    assert Role.objects.filter(id=role.id).count() == 0
    assert Invitation.objects.filter(id=invitation.id).count() == 0
    assert InvitationLink.objects.filter(invitation__id=invitation.id).count() == 0


@pytest.mark.django_db
def test_invite_resend_existing_invite_active(
    client,
    log_in,
    user_sponsor,
    person_invited,
    invitation,
    invitation_link,
    invitation_link_expired,
    mocker,
):
    send_invite_mock_function = mocker.patch(
        "gregui.api.views.invitation.send_invite_mail"
    )
    log_in(user_sponsor)

    invitation_links_for_person = InvitationLink.objects.filter(
        invitation__role__person_id=person_invited.id
    )
    assert invitation_links_for_person.count() == 2

    original_expire_date = invitation_link_expired.expire
    original_date = invitation_link.expire
    url = reverse("gregui-v1:invite-resend", kwargs={"person_id": person_invited.id})
    response = client.patch(url)
    assert response.status_code == status.HTTP_200_OK

    # One e-mail should have been sent with an invite
    assert send_invite_mock_function.call_count == 1

    # Just check that the expire dates have not changed
    invitation_link_expired.refresh_from_db()
    assert original_expire_date == invitation_link_expired.expire

    invitation_link.refresh_from_db()
    assert original_date == invitation_link.expire

    # The number of invitations should not have changed
    invitation_links_for_person = InvitationLink.objects.filter(
        invitation__role__person_id=person_invited.id
    )
    assert invitation_links_for_person.count() == 2


@pytest.mark.django_db
def test_invite_resend_existing_invite_not_active(
    client, log_in, user_sponsor, person_invited, invitation_link_expired, mocker
):
    send_invite_mock_function = mocker.patch(
        "gregui.api.views.invitation.send_invite_mail", return_value=10
    )
    log_in(user_sponsor)

    invitation_links_for_person = InvitationLink.objects.filter(
        invitation__role__person_id=person_invited.id
    )
    assert invitation_links_for_person.count() == 1

    original_expire_date = invitation_link_expired.expire
    url = reverse("gregui-v1:invite-resend", kwargs={"person_id": person_invited.id})
    response = client.patch(url)
    assert response.status_code == status.HTTP_200_OK

    # One e-mail should have been sent with an invite
    assert send_invite_mock_function.call_count == 1

    # Just check that the expire dates have not changed
    invitation_link_expired.refresh_from_db()
    assert original_expire_date == invitation_link_expired.expire

    # There should now be two invitations
    invitation_links_for_person = InvitationLink.objects.filter(
        invitation__role__person_id=person_invited.id
    )
    assert invitation_links_for_person.count() == 2

    # If there is no active link the following line will raise an exception
    InvitationLink.objects.get(
        invitation__role__person_id=person_invited.id, expire__gt=timezone.now()
    )
